use crate::domain::{Context, Ingredient};
use crate::repo::ingredient::IngredientRepo;

#[derive(Default, Debug)]
pub struct RequestOpts {
    pub keys: Option<Vec<String>>,
}

pub fn execute(repo: &impl IngredientRepo, ctx: &Context, opts: RequestOpts) -> Vec<Ingredient> {
    repo.get_ingredients(ctx, opts.into())
}

#[cfg(test)]
mod tests {
    use super::*;
    use crate::domain::Lang;
    use crate::repo::ingredient::InMemoryIngredientRepo;

    #[test]
    fn should_return_all_ingredients() {
        let repo = InMemoryIngredientRepo::new();
        let res = execute(&repo, &Context::default(), RequestOpts::default());

        match res.as_slice() {
            [banana, apple, orange, salt, sugar] => {
                assert_eq!(banana.key, String::from("banana"));
                assert_eq!(banana.lang, Lang::Rus);
                assert_eq!(banana.name, String::from("Банан"));

                assert_eq!(apple.key, String::from("apple"));
                assert_eq!(apple.lang, Lang::Rus);
                assert_eq!(apple.name, String::from("Яблоко"));

                assert_eq!(orange.key, String::from("orange"));
                assert_eq!(orange.lang, Lang::Rus);
                assert_eq!(orange.name, String::from("Апельсин"));

                assert_eq!(salt.key, String::from("salt"));
                assert_eq!(salt.lang, Lang::Rus);
                assert_eq!(salt.name, String::from("Соль"));

                assert_eq!(sugar.key, String::from("sugar"));
                assert_eq!(sugar.lang, Lang::Rus);
                assert_eq!(sugar.name, String::from("Сахар"));
            }
            _ => unimplemented!(),
        }
    }

    #[test]
    fn should_return_preferred_lang_with_fallback() {
        let repo = InMemoryIngredientRepo::new();
        let res = execute(&repo, &Context::eng(), RequestOpts::default());

        match res.as_slice() {
            [banana, apple, orange, salt, sugar] => {
                assert_eq!(banana.key, String::from("banana"));
                assert_eq!(banana.lang, Lang::Eng);
                assert_eq!(banana.name, String::from("Banana"));

                assert_eq!(apple.key, String::from("apple"));
                assert_eq!(apple.lang, Lang::Eng);
                assert_eq!(apple.name, String::from("Apple"));

                assert_eq!(orange.key, String::from("orange"));
                assert_eq!(orange.lang, Lang::Rus);
                assert_eq!(orange.name, String::from("Апельсин"));

                assert_eq!(salt.key, String::from("salt"));
                assert_eq!(salt.lang, Lang::Eng);
                assert_eq!(salt.name, String::from("Salt"));

                assert_eq!(sugar.key, String::from("sugar"));
                assert_eq!(sugar.lang, Lang::Rus);
                assert_eq!(sugar.name, String::from("Сахар"));
            }
            _ => unimplemented!(),
        }
    }

    #[test]
    fn should_return_filtered_by_keys() {
        let repo = InMemoryIngredientRepo::new();
        let res = execute(
            &repo,
            &Context::default(),
            RequestOpts {
                keys: Some(vec![String::from("apple")]),
            },
        );

        match res.as_slice() {
            [apple] => {
                assert_eq!(apple.key, String::from("apple"));
                assert_eq!(apple.lang, Lang::Rus);
                assert_eq!(apple.name, String::from("Яблоко"));
            }
            _ => unimplemented!(),
        }
    }
}
